<?php
namespace Reaktion\Common;

abstract class OrderScript
{
    use DisplaysScripts;

    /** @var bool */
    private $handled = false;

    /** @var int $order_id */
    abstract public function handle($order_id);

    public function __construct()
    {
        add_action('woocommerce_thankyou', [$this, 'handleHook']);
        add_action('wp_enqueue_scripts', [$this, 'handleFallback']);
    }

    /** @var int $order_id */
    public function handleHook($order_id)
    {
        if (!$this->shouldHandle()) {
            return;
        }
        $this->handle($order_id);
    }

    /** `woocommerce_thankyou` hook is not always called, so a fallback is needed. */
    public function handleFallback()
    {
        if (!$this->shouldHandle()) {
            return;
        }
        $order_id =
            get_query_var('order-received') ?: wc_clean($_GET['order_id']);
        if (!$order_id) {
            return;
        }
        $order = wc_get_order($order_id);
        if (!$order) {
            return;
        }
        $order_key = wc_clean($_GET['key']);
        if ($order->get_order_key() !== $order_key) {
            return;
        }
        $this->handle($order_id);
    }

    /**
     * Ensures that a script is handled only once.
     * @return bool
     */
    private function shouldHandle()
    {
        if ($this->handled) {
            return false;
        }
        return $this->handled = true;
    }
}
