<?php
namespace Reaktion\Sessions;

use Reaktion\Common\ChecksContext;
use Reaktion\Common\ChecksTrackingConsent;
use Reaktion\Common\ManagesCookies;

class CartIdCookie
{
    use ChecksContext, ChecksTrackingConsent, ManagesCookies;

    const COOKIE_NAME = 'reaktion_cart_id';
    const META_DATA_KEY = '_reaktion_cart_id';

    public function __construct()
    {
        if (!$this->is_frontend() || !$this->has_tracking_consent()) {
            return;
        }
        add_action('init', [$this, 'initialize_cookie']);
        add_action(
            'woocommerce_checkout_create_order',
            [$this, 'append_order_metadata'],
            20,
            2
        );
    }

    public function initialize_cookie()
    {
        if (!$this->readCookie(static::COOKIE_NAME)) {
            $value = $this->generateCartId();
            $this->writeCookie(static::COOKIE_NAME, $value);
        }
    }

    /** @param WC_Order $order */
    public function append_order_metadata($order, $data)
    {
        $value = $this->readCookie(static::COOKIE_NAME);
        if (empty($value)) {
            return;
        }
        $order->update_meta_data(static::META_DATA_KEY, $value);
        $this->writeCookie(static::COOKIE_NAME, '');
    }

    /** @return string */
    private function generateCartId()
    {
        return wc_rand_hash();
    }
}
