<?php
namespace Reaktion\Sessions;

use Reaktion\Common\ChecksContext;
use Reaktion\Common\ChecksTrackingConsent;
use Reaktion\Common\ManagesCookies;

class SessionIdCookie
{
    use ChecksContext, ChecksTrackingConsent, ManagesCookies;

    const COOKIE_NAME = 'reaktion_session_id';

    public function __construct()
    {
        if (!$this->is_frontend() || !$this->has_tracking_consent()) {
            return;
        }
        add_action('init', [$this, 'initialize_cookie']);
    }

    public function initialize_cookie()
    {
        if (!$this->readCookie(static::COOKIE_NAME)) {
            $value = $this->generate_session_id();
            $this->writeCookie(static::COOKIE_NAME, $value);
        }
    }

    /** @return string */
    private function generate_session_id()
    {
        return substr(wc_rand_hash() . wc_rand_hash(), 0, 64);
    }
}
